@@ -1,10 +1,12 @@
-# This file is Copyright (c) 2014 Yann Sionneau <ys@m-labs.hk>
-# This file is Copyright (c) 2014-2018 Florent Kermarrec <florent@enjoy-digital.fr>
-# This file is Copyright (c) 2013-2014 Robert Jordens <jordens@gmail.com>
-# This file is Copyright (c) 2015-2014 Sebastien Bourdeauducq <sb@m-labs.hk>
-# This file is Copyright (c) 2020 Antmicro <www.antmicro.com>
-
-# License: BSD
+#
+# This file is part of LiteX.
+#
+# Copyright (c) 2014 Yann Sionneau <ys@m-labs.hk>
+# Copyright (c) 2014-2018 Florent Kermarrec <florent@enjoy-digital.fr>
+# Copyright (c) 2013-2014 Robert Jordens <jordens@gmail.com>
+# Copyright (c) 2015-2014 Sebastien Bourdeauducq <sb@m-labs.hk>
+# Copyright (c) 2020 Antmicro <www.antmicro.com>
+# SPDX-License-Identifier: BSD-2-Clause
 
 
 from migen import *
@@ -127,11 +129,13 @@ class SpiFlashDualQuad(SpiFlashCommon, AutoCSR):
         addr_width = 24
 
         dq = TSTriple(spi_width)
-        # Keep DQ2,DQ3 as outputs during bitbang, this ensures they activate ~WP or ~HOLD functions
         self.specials.dq0 = Tristate(pads.dq[0], o=dq.o[0], i=dq.i[0], oe=dq.oe)
         self.specials.dq1 = Tristate(pads.dq[1], o=dq.o[1], i=dq.i[1], oe=dq.oe)
-        self.specials.dq2 = Tristate(pads.dq[2], o=dq.o[2], i=dq.i[2], oe=(dq.oe | self.bitbang_en.storage))
-        self.specials.dq3 = Tristate(pads.dq[3], o=dq.o[3], i=dq.i[3], oe=(dq.oe | self.bitbang_en.storage))
+        if spi_width > 2:
+            # Keep DQ2,DQ3 as outputs during bitbang, this ensures they activate ~WP or ~HOLD functions
+            bitbang_en = 0 if not with_bitbang else self.bitbang_en.storage
+            self.specials.dq2 = Tristate(pads.dq[2], o=dq.o[2], i=dq.i[2], oe=(dq.oe | bitbang_en))
+            self.specials.dq3 = Tristate(pads.dq[3], o=dq.o[3], i=dq.i[3], oe=(dq.oe | bitbang_en))
 
         sr = Signal(max(cmd_width, addr_width, wbone_width))
         if endianness == "big":
